#include "i0mapmontopoobjfilter.h"
#include "i0mmtopobarchart.h"

#include <qwt_plot_canvas.h>
#include <qwt_series_data.h>
#include <qwt_plot_marker.h>
#include <qwt_plot_multi_barchart.h>
#include <qwt_plot.h>
#include <QMouseEvent>

bool I0MapMonObjFilter::eventFilter(QObject *watched, QEvent *event) {
    if(event->type() == QEvent::MouseButtonPress && static_cast<QMouseEvent *>(event)->button() == Qt::LeftButton) {
        QwtPlotCanvas *canvas = static_cast<QwtPlotCanvas *>(watched);
        QwtPlot *plot = canvas->plot();
        bool stacked = static_cast<I0MMTopoBarChart *>(plot)->stackedMode();
        // marker visible? hide it
        QwtPlotMarker *marker = nullptr;
        for(int i = 0; i < plot->itemList().size() && marker == nullptr; i++) {
            if(plot->itemList().at(i)->rtti() == QwtPlotItem::Rtti_PlotMarker) {
                marker = static_cast<QwtPlotMarker *>(plot->itemList().at(i));
                if(marker->title().text() != "intensity")
                    marker = nullptr;
            }
        }
        if(marker)
            delete marker;
        QwtPlotMultiBarChart* bchart = nullptr;
        for(int i = 0; i < plot->itemList().size() && bchart == nullptr; i++) {
            if(plot->itemList().at(i)->rtti() == QwtPlotItem::Rtti_PlotMultiBarChart) {
                double y = plot->invTransform(QwtPlot::yLeft, static_cast<QMouseEvent *>(event)->pos().y());
                size_t idx = qRound(y);
                bchart = static_cast<QwtPlotMultiBarChart *>(plot->itemList().at(i));
                const QwtSeriesData< QwtSetSample > * dat = bchart->data ();
                printf("ClickEventFilter::eventFilter: \e[1;36mx is %f idx %d data siz %ld\e[0m\n", y, qRound(y), dat->size());
                if(idx < dat->size()) {
                    QVector<double> values = dat->sample(idx).set;
                    if(values.size() > 0) {
                        const QStringList& nms = static_cast<NamesAxisDraw *>(plot->axisScaleDraw(QwtPlot::yLeft))->names();
                        const QString& xn = nms.size() > (int) idx ? nms[idx] : "-";
                        QString ys;
                        double xsum = 0;
                        for(int j = 0; j < values.size(); j++) {
                            ys += QString("λ %1: %2").arg(j+1).arg(values[j]) + "\n";
                            if(stacked) xsum += values[j];
                            else if(fabs(values[j]) > xsum) xsum = values[j];
                        }
                        double xlb = plot->axisScaleDiv(QwtPlot::xBottom).lowerBound();
                        double xub = plot->axisScaleDiv(QwtPlot::xBottom).upperBound();
                        double d1 = 1.0 / 4.0 * (xub - xlb), d2 = 3.0 * d1;
//                        double y = ysum - ylb;
                        double x = xsum;
                        QwtText lab = xn + "\n" + ys;
                        lab.setBackgroundBrush(QColor("Linen"));
                        marker = new QwtPlotMarker();
                        marker->setTitle("intensity");
                        marker->attach(plot);
                        marker->show();
                        marker->setYValue(idx);
                        marker->setLabel(lab);
                        if(idx >= dat->size() / 2.5 ) // axis scale engine is inverted
                            marker->setLabelAlignment(Qt::AlignTop);
                        else
                            marker->setLabelAlignment(Qt::AlignBottom);
                        if(x < d1 ) marker->setLabelAlignment(marker->labelAlignment() | Qt::AlignRight);
                        else if(x > d2) marker->setLabelAlignment(marker->labelAlignment() | Qt::AlignLeft);
                        marker->setXValue(xsum);

                        plot->replot();
                    }
                }
            }
        }

        return true;
    }
    return QObject::eventFilter(watched, event);
}
